#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <getopt.h>
#include <limits.h>

#define PACKAGENAME "i2bits"
#define VERSION     "0.0.2"

static void print_bits(uint32_t);
static void print_help(FILE *);
static int print_int(const char *, int);

int main(int argc, char **argv)
{
    uint32_t num = 0;
    int base = 10;
    const char *input;
    int opt;

    enum {
        HELP_OPTION = CHAR_MAX + 1
    };

    static const struct option longopts[] = {
        { "bits2d",  required_argument, NULL, 'd'         },
        { "bits2h",  required_argument, NULL, 'h'         },
        { "version", no_argument,       NULL, 'V'         },
        { "help",    no_argument,       NULL, HELP_OPTION },
        {  NULL,     0,                 NULL,  0          }
    };

    if (argc == 1)
    {
        print_help(stderr);
        return 1;
    }

    while ((opt = getopt_long(argc, argv, "h:d:V", longopts, NULL)) != -1)
    {
         switch (opt) {
         case 'h':
             return print_int(optarg, 16);

         case 'd':
             return print_int(optarg, 10);

         case 'V':
             printf("%s %s\n", PACKAGENAME, VERSION);
             return 0;

         case HELP_OPTION:
             print_help(stdout);
             return 0;

         default:
             print_help(stderr);
             return 1;
         }
    }

    input = argv[1];

    if (*input == '0' && *(input + 1) == 'x')
        base = 16;

    num = strtoul(input, NULL, base);

    print_bits(num);

    return 0;
}

static void print_bits(uint32_t num)
{
    int n;

    for (n = 31; n >= 0; n--)
    {
        (num & (1 << n)) ? putchar('1') : putchar('0');

        if ((n % 8 == 0) && (n != 0))
            putchar('.');
    }

    putchar('\n');
}

static int print_int(const char *bits, int base)
{
    uint32_t res = 0;
    size_t blen, idx, n;

    blen = strlen(bits);
    idx = blen - 1;

    if (blen > 32)
    {
        fprintf(stderr, "error: bits length must be <= 32\n");
        return 1;
    }

    for (n = 0; n != blen; n++, idx--)
    {
        if (*(bits + idx) == 0x31)      /* 1 */
            res += 1 << n;
        else if (*(bits + idx) == 0x30) /* 0 */
            continue;
        else
        {
            fprintf(stderr, "error: incorrect bit: %c\n", *(bits + idx));
            return 1;
        }
    }

    switch (base) {
    case 10:
        printf("%d\n", res);
        break;
    case 16:
        printf("0x%x\n", res);
        break;
    default:
        fprintf(stderr, "Unsupported base\n");
        return 1;
    }

    return 0;
}

static void print_help(FILE *out)
{
    fprintf(out, "Usage:\n  %s <decimal|hexadecimal number>\n\n", PACKAGENAME);
    fputs("Options:\n", out);
    fputs(" -d bits, --bits2d: convert bits to decimal\n", out);
    fputs(" -h bits, --bits2h: convert bits to hexadecimal\n", out);
    fputs(" -V,   --version:   print version\n", out);
    fputs("       --help:      print help and exit\n", out);
}
